unit oDBConnection;

interface

uses
  Dialogs,
  Classes,

  TRPCB,
  SysUtils
  , iMDK_DBConnection
  , oInterfaces
  ;

type
  TDBConnection = class(TCustomInterfacedObject,i_RPCExecutor)
  private
    fRPCBroker: TRPCBroker;
    fRPCBrokerResults: TStringList;
    fRPCLog: TStringList;
    procedure DecodeResults;
  public
    property ExecResults: TStringList
      read fRPCBrokerResults;

    constructor Create;
    destructor Destroy; override;
    function IsConnected: boolean;
    function Connect(aParams: array of string): boolean;
    function ExecRemoteProc(
      aRPCName: string;
      aParams: array of string;
      aBigString: string): boolean; overload;
    function ExecRemoteProc(
      aRPCName: string;
      aParams: array of string;
      aMultList: TStringList): boolean; overload;
    function getExecResults: TStringList;
  end;

function NewDBConnection(aRPCBroker: TRPCBroker): TDBConnection;

implementation

uses
  IdCoderUUE
  , uMDK_RPCEvent
  ;

const
  UUE_HEADER = '-- Begin UUEncoded Data --';
  UUE_FOOTER = '--- End UUEncoded Data ---';
  MAX_RPC_LINES = 100;

function NewDBConnection(aRPCBroker: TRPCBroker): TDBConnection;
begin
  Result := TDBConnection.Create;
  if Assigned(aRPCBroker) then
    Result.fRPCBroker := aRPCBroker
  else
    Result.fRPCBroker := TRPCBroker.Create(nil);
end;

{ TDBConnection }

constructor TDBConnection.Create;
begin
  inherited;
  fRPCBroker := nil;
  fRPCBrokerResults := TStringList.Create;
  fRPCLog := TStringList.Create;
end;

destructor TDBConnection.Destroy;
begin
  fRPCBroker := nil;
  FreeAndNil(fRPCBrokerResults);
  while fRPCLog.Count > 0 do
  begin
    try
      if fRPCLog.Objects[0] <> nil then
        TRPCEventItem(fRPCLog.Objects[0]).Free;
      fRPCLog.Delete(0);
    except
    end;
  end;
  FreeAndNil(fRPCLog);


  inherited;
end;

function TDBConnection.Connect(aParams: array of string): boolean;
begin
  try
    fRPCBroker.Server := aParams[0];
    fRPCBroker.ListenerPort := StrToInt(aParams[1]);
    if Length(aParams) > 2 then
      fRPCBroker.AccessVerifyCodes := aParams[2];
    fRPCBroker.Connected := True;
    Result := fRPCBroker.Connected;
  except
    Result := False;
  end;
end;

function TDBConnection.IsConnected: boolean;
begin
  Result := fRPCBroker.Connected;
end;

procedure TDBConnection.DecodeResults;
{Scan the fRPCBrokerResults for and encoded data and set it back to the way it was sent}
var
  i: integer;
{$IFDEF D10}
  s,ss: String;
{$ELSE}
  aStringStream: TStringStream;
{$ENDIF}
begin
  // if the data is encoded both header and footer should be present
  //                                             zzzzzzandria 060721
  if (pos(UUE_HEADER, fRPCBrokerResults.Text) = 0)
    or (pos(UUE_FOOTER, fRPCBrokerResults.Text) = 0) then
    Exit;

  for i := fRPCBrokerResults.Count - 1 downto 0 do
    if fRPCBrokerResults[i] = UUE_HEADER then
      begin
{$IFDEF D10}
        s := '';
{$ELSE}
        aStringStream := TStringStream.Create('');
{$ENDIF}
        with TIdDecoderUUE.Create(nil) do
          try
{$IFDEF D10}
            ss := '';
{$ELSE}
            aStringStream.Position := 0;
{$ENDIF}
            while fRPCBrokerResults[i + 1] <> UUE_FOOTER do
              begin
{$IFDEF D10}
                ss := fRPCBrokerResults[i + 1];
                ss := DecodeString(ss);
                s := s + ss;
{$ELSE}
                DecodeToStream(fRPCBrokerResults[i + 1], aStringStream);
{$ENDIF}
                fRPCBrokerResults.Delete(i + 1);
              end;
            fRPCBrokerResults.Delete(i + 1);
{$IFDEF D10}
            fRPCBrokerResults[i] := s;
{$ELSE}
            aStringStream.Seek(0, soFromBeginning);
            fRPCBrokerResults[i] := aStringStream.DataString;
{$ENDIF}
            {Replace header with original text}
          finally
{$IFNDEF D10}
            FreeAndNil(aStringStream);
{$ENDIF}
            Free;
          end;
      end;
end;

function TDBConnection.ExecRemoteProc(
  aRPCName: string;
  aParams: array of string;
  aMultList: TStringList): Boolean;
var
  i, j: integer;
begin
  fRPCBroker.RemoteProcedure := aRPCName;
  {Load any params}
  i := 0;
  if SizeOf(aParams) > 0 then
    while i <= High(aParams) do
      begin
        if (Copy(aParams[i], 1, 1) = '@') and (Length(aParams[i]) > 1) then
          // Is it passed as a reference???
          begin
            fRPCBroker.Param[i].PType := reference;
            fRPCBroker.Param[i].Value := Copy(aParams[i], 2,
              Length(aParams[i]));
          end
        else
          begin
            fRPCBroker.Param[i].PType := literal;
            fRPCBroker.Param[i].Value := aParams[i];
          end;
        inc(i);
      end;

  {Load up the multiple if it is assigned}
  if Assigned(aMultList) then
    begin
      fRPCBroker.Param[i].PType := list;
      if fRPCBroker.Param[i].Mult.Count > 0 then
        ShowMessage('Broker Param list was not cleaned.' + #13#10 +
          'Possible connection failure...' + #13#10 +
          'Close the application and try again later');

      for j := 0 to aMultList.Count - 1 do
        fRPCBroker.Param[i].Mult[IntToStr(j)] := aMultList[j];

    end;

  try
    fRPCBrokerResults.Clear;
    fRPCBroker.Results.Clear; // 061129
    fRPCBroker.lstCall(fRPCBrokerResults);
    DecodeResults; {Checks for any UUEncode data coming back}
    Result := (Copy(fRPCBrokerResults.Text, 1, 2) <> '-1');
    {Check for error flag}
  except
    on e: EBrokerError do
      begin
        fRPCBrokerResults.Insert(0, '-1^' + e.Message);
        Result := False;
      end;
  else
    raise;
  end;
end;

function TDBConnection.ExecRemoteProc(
  aRPCName: string;
  aParams: array of string;
  aBigString: string): boolean;
var
  aStrStream: TStringStream;
  aStrList: TStringList;
  aStrListPartial: TStringList; {This is used to send data down in chunks}

begin
  {The purpose here is to force a complex string into UUEncode before calling above}
  aStrStream := TStringStream.Create(aBigString);
  aStrList := TStringList.Create;
  //  aStrListPartial := TStringList.Create;  // 061129 zzzzzzandria

  with TIdEncoderUUE.Create(nil) do
    try
      aStrList.Add(UUE_HEADER);
      aStrStream.Position := 0;
      while aStrStream.Position < aStrStream.Size do
        aStrList.Add(Encode(aStrStream, 45));
      aStrList.Add(UUE_FOOTER);
    finally
      FreeAndNil(aStrStream);
      Free;
    end;

  {Now we have a UUEncoded TStringList instead of aBigString}
  if aStrList.Count < MAX_RPC_LINES then {We can safely send this tiny list to M}
    begin
      Result := ExecRemoteProc(aRPCName, aParams, aStrList);
      FreeAndNil(aStrList);
    end
  else
    try {Create partial list object and do it in pieces}
      aStrListPartial := TStringList.Create;
      Result := True;
      while aStrList.Count > 0 do
        begin
          {Move MAX_RPC_LINES lines to smaller array}
          aStrListPartial.Clear;
          while (aStrListPartial.Count < MAX_RPC_LINES) and (aStrList.Count > 0)
            do
            begin
              aStrListPartial.Add(aStrList[0]);
              aStrList.Delete(0);
            end;

          {Call the RPC with the smaller list}
          Result := ExecRemoteProc(aRPCName, aParams, aStrListPartial);

        end;
      FreeAndNil(aStrListPartial);
    except
      FreeAndNil(aStrListPartial);
      Result := False;
    end;
end;

function TDBConnection.getExecResults: TStringList;
begin
  Result := fRPCBrokerResults;
end;

(*
function CallRPC(RemoteProcedure: string;
  Parameters: array of string; MultList: TStringList = nil;
  RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
var
  anItem: TRPCEventItem;
  aStart, aStop: TDateTime;
  aList: TStrings;
  iLimit: Integer;
  i: integer;
  SL: TStringList;
begin
  aStart := Now;
  Result :=
    CallRemoteProc(
    RPCBroker,
    RemoteProcedure, Parameters, MultList, RPCMode, RetList);
  aStop := Now;

  if RetList = nil then aList := RPCBroker.Results
  else aList := RetList;

  anItem := getRPCEventItem(aStart, aStop,
    RemoteProcedure,
    Parameters,
    MultList,
    RPCMode,
    aList
    );

  SL := TStringList.Create;
  for i := Low(parameters) to High(Parameters) do
    SL.Add(Parameters[i]);

  Inc(RPCCount);
  RPCLog.InsertObject(0,
    Format('%10.10d ', [RPCCount]) +
    FormatDateTime('hh:mm:ss.zzz', aStart) + '   ' + RemoteProcedure
    + ' (' + SL.CommaText + ')' // zzzzzzandria 060724
    ,
    anItem);
  SL.Free;

  iLimit := 300;
  if assigned(frmGMV_RPCLog) then
    iLimit := StrToIntDef(frmGMV_RPCLog.ComboBox1.Text, iLimit);

  while RPCLog.Count > iLimit do
  begin
    if RPCLog.Objects[RPCLog.Count - 1] <> nil then
      TRPCEventItem(RPCLog.Objects[RPCLog.Count - 1]).Free;
    RPCLog.Delete(RPCLog.Count - 1);
  end;

  if assigned(frmGMV_RPCLog) then
  begin
    frmGMV_RPCLog.lbLog.Items.Assign(RPCLog);
    frmGMV_RPCLog.lbLog.ItemIndex := 0;
    frmGMV_RPCLog.lbLogClick(nil);
  end;
end;
*)

end.



